"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const buildRawPath = (map, name) => {
    const route = map.get(name);
    if (!route) {
        throw new Error(`Undefined route "${name}"`);
    }
    if (!route.path) {
        throw new Error(`Route "${name}" does not have a path`);
    }
    return route.path;
};
// eslint-disable-next-line @typescript-eslint/no-explicit-any
exports.buildRoutePath = (map, name, params = {}) => (buildRawPath(map, name)
    .replace(/:\w+\??/g, (match) => {
    const paramKey = match.replace(/[:?]/g, '');
    const paramValue = params[paramKey];
    if (!paramValue && !match.includes('?')) {
        throw new Error(`Missing value for required param "${paramKey}"`);
    }
    return paramValue || '';
})
    .replace(/\/$/, ''));
exports.mapRoutes = (routes, basename = '', parents = [], map = new Map()) => {
    routes.forEach((route) => {
        if (route.routes) {
            exports.mapRoutes(route.routes, basename, route.name ? [...parents, route.name] : parents, map);
        }
        if (!route.name && route.path) {
            throw new Error(`Route ${route.path} has no name`);
        }
        if (!route.name) {
            return;
        }
        const existingRoute = map.get(route.name);
        if (process.env.NODE_ENV !== 'production' && existingRoute) {
            // eslint-disable-next-line no-console
            console.warn(`Duplicate definition for route ${route.name}:\n- ${existingRoute.path}\n- ${route.path}`);
        }
        map.set(route.name, Object.assign({}, route, { parents }, (route.path ? {
            regex: new RegExp(`^${(basename + route.path).replace(/\/:\w+\?/g, '/?([\\w_-]*)').replace(/:\w+/g, '([\\w_-]+)')}\\/?$`),
        } : {})));
    });
    return map;
};
class RoutingContext {
    constructor(routesMap, routerContext) {
        this.match = (pathname) => Array.from(this.routesMap.values())
            .find(route => !!route.regex && route.regex.test(pathname)) || null;
        this.getPath = (name, params) => exports.buildRoutePath(this.routesMap, name, params);
        this.getRawPath = (name) => buildRawPath(this.routesMap, name);
        this.getRoute = (name) => {
            const route = this.routesMap.get(name);
            if (!route) {
                throw new Error(`Undefined route "${name}"`);
            }
            return route;
        };
        this.push = (name, params) => this.history
            .push(exports.buildRoutePath(this.routesMap, name, params));
        this.replace = (name, params) => this.history
            .replace(exports.buildRoutePath(this.routesMap, name, params));
        this.routesMap = routesMap;
        this.routerContext = routerContext;
    }
    get location() {
        return this.routerContext.location;
    }
    get history() {
        if (!this.routerContext.history) {
            throw new Error('History API cannot be used on server side');
        }
        return this.routerContext.history;
    }
    get params() {
        return this.routerContext.match ? this.routerContext.match.params : {};
    }
}
exports.RoutingContext = RoutingContext;
exports.buildRoutingContext = (routes, routerContext, basename) => {
    const map = exports.mapRoutes(routes, basename);
    let finalContext = routerContext;
    if (typeof routerContext.location === 'string') {
        const [pathname, search] = routerContext.location.split('?');
        finalContext = Object.assign({}, routerContext, { location: {
                pathname,
                search: search ? `?${search}` : '',
                state: {},
                hash: '',
            } });
    }
    return new RoutingContext(map, finalContext);
};
